import configparser
import logging
import webbrowser

import requests
from ratelimit import limits, sleep_and_retry
from requests_oauthlib import OAuth2Session

from .teamleaderObjects import *

REDIRECT_URI = "https://localhost:3000/oauth.php"
TOKEN_SITE = "https://app.teamleader.eu/oauth2/access_token"
AUTHORIZE_URI = "https://app.teamleader.eu/oauth2/authorize"
REFRESH_URL = "https://app.teamleader.eu/oauth2/access_token"


class Client:
    def __init__(
        self,
        client_id=None,
        client_secret=None,
        teamleader_token_file_name=None,
        config_file_path=None,
    ):
        if config_file_path:
            config = configparser.ConfigParser()
            config_file_path = os.path.abspath(config_file_path)
            if not os.path.isfile(config_file_path):
                raise FileNotFoundError("Config file not found")
            config.read(config_file_path)
            client_id = config["teamleader"]["CLIENT_ID"]
            client_secret = config["teamleader"]["CLIENT_SECRET"]
            teamleader_token_file_name = config["teamleader"]["TOKEN_FILE_PATH"]

        if not (client_id and client_secret and teamleader_token_file_name):
            raise ValueError("All parameters should be filled")

        self.teamleader_token_file_name = teamleader_token_file_name
        self.client = OAuth2Session(
            client_id,
            token=self._get_token(client_id=client_id, client_secret=client_secret),
            auto_refresh_url=REFRESH_URL,
            auto_refresh_kwargs={
                "client_id": client_id,
                "client_secret": client_secret,
            },
            token_updater=self._token_saver,
        )
        self.departments = Departments(self.get_teamleader, self.post_teamleader)
        self.users = Users(self.get_teamleader, self.post_teamleader)
        self.projects = Projects(self.get_teamleader, self.post_teamleader)
        self.custom_fields = CustomFields(self.get_teamleader, self.post_teamleader)
        self.worktypes = WorkTypes(self.get_teamleader, self.post_teamleader)
        self.contacts = Contacts(self.get_teamleader, self.post_teamleader)
        self.companies = Companies(self.get_teamleader, self.post_teamleader)
        self.businesstypes = BusinessTypes(self.get_teamleader, self.post_teamleader)
        self.tags = Tags(self.get_teamleader, self.post_teamleader)
        self.dealphases = DealPhases(self.get_teamleader, self.post_teamleader)
        self.dealsources = DealSources(self.get_teamleader, self.post_teamleader)
        self.tasks = Tasks(self.get_teamleader, self.post_teamleader)
        self.invoices = Invoices(self.get_teamleader, self.post_teamleader)
        self.events = Events(self.get_teamleader, self.post_teamleader)
        self.webhooks = Webhooks(self.get_teamleader, self.post_teamleader)
        self.timetracking = TimeTracking(self.get_teamleader, self.post_teamleader)
        self.migrate = Migrate(self.get_teamleader, self.post_teamleader)
        self.deals = Deals(self.get_teamleader, self.post_teamleader)
        self.lostreasons = LostReasons(self.get_teamleader, self.post_teamleader)
        self.teams = Teams(self.get_teamleader, self.post_teamleader)

    def _get_token(self, client_id, client_secret):
        try:
            return self._token_open()
        except FileNotFoundError:
            return self._request_new_token(client_id, client_secret)

    def _token_saver(self, local_token):
        with open(self.teamleader_token_file_name, "wb") as output:
            pickle.dump(local_token, output, protocol=5)
        print("token saved")

    def _token_open(self):
        with open(self.teamleader_token_file_name, "rb") as input:
            return pickle.load(input)

    def _request_new_token(self, client_id, client_secret):
        oauth = OAuth2Session(client_id, redirect_uri=REDIRECT_URI)
        authorization_url, _ = oauth.authorization_url(
            AUTHORIZE_URI,
        )
        webbrowser.open_new(authorization_url)
        authorization_response = input("Enter the full callback URL")
        return oauth.fetch_token(
            TOKEN_SITE,
            authorization_response=authorization_response,
            client_id=client_id,
            client_secret=client_secret,
        )

    @sleep_and_retry
    @limits(calls=100, period=60)
    def _request_method(self, method, url_addition, *args, **kwargs):
        response = method("https://api.teamleader.eu/" + url_addition, *args, **kwargs)
        try:
            response.raise_for_status()
        except requests.HTTPError as e:
            logging.info(response.json())
            raise e
        finally:
            return response

    def get_teamleader(self, url_addition, *args, **kwargs):
        return self._request_method(self.client.get, url_addition, *args, **kwargs)

    def post_teamleader(self, url_addition, *args, **kwargs):
        return self._request_method(self.client.post, url_addition, *args, **kwargs)



import json
import os
import pickle
import time
from itertools import count

from .teamleaderStores import Webhook


class TeamleaderFileHandler:
    file = {}

    def __init__(self, object_name, refresh_function):
        if not os.path.exists("object_cache"):
            os.makedirs("object_cache")
        self.file_path = "object_cache/" + object_name
        file_exists = os.path.isfile(self.file_path)
        time_elapsed = -1
        if file_exists:
            self.file = self.file_loader()
            time_elapsed = time.time() - self.file["time"]

        if not file_exists or time_elapsed > 60 * 60 * 24:
            self.file["time"] = time.time()
            self.file["file"] = refresh_function()
            self.file_saver(self.file)
            print(f"File '{object_name}' refreshed")

    def file_saver(self, json_file):
        with open(self.file_path, "wb+") as output:
            pickle.dump(json_file, output, pickle.HIGHEST_PROTOCOL)

    def file_loader(self):
        with open(self.file_path, "rb") as input:
            return pickle.load(input)


class TeamleaderTag:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def tag(self, object_id, tags):
        if not isinstance(tags, list):
            tags = [tags]
        data = {"id": object_id, "tags": tags}
        return self.post(self.url + ".tag", data=json.dumps(data))


class TeamleaderUntag:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def untag(self, object_id, tags):
        if not isinstance(tags, list):
            tags = [tags]
        data = {"id": object_id, "tags": tags}
        return self.post(self.url + ".untag", data=json.dumps(data))


class TeamleaderPagesList:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def list(self, data={}, feedback=False, sideloading=""):
        if sideloading:
            data.update({"include": sideloading})
        there_are_more_pages = True
        c = count(1)
        size = 50
        while there_are_more_pages:
            number = next(c)
            if feedback:
                print(f"page: {number}, size:{size}, total:{number * size}")
            page_data = {"page": {"size": size, "number": number}}
            data.update(page_data)
            respons = self.post(self.url + ".list", data=json.dumps(data))
            object_lists = respons.json().get("data")
            included = respons.json().get("included", {})
            if not object_lists:
                return
            there_are_more_pages = len(object_lists) == size
            for object_element in object_lists:
                if included:
                    for key, value in included.items():
                        current_value = object_element.get(key)
                        if current_value:
                            replacement_value = self.get_right_element(
                                value, current_value
                            )
                            object_element[key] = replacement_value
                yield object_element

    def get_right_element(self, value, current_value):
        for element in value:
            if current_value["id"] == element["id"]:
                return element


class TeamleaderSimpleList:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def list(self, data={}):
        return self.get(self.url + ".list", data=json.dumps(data)).json()


class TeamleaderCustomField:
    value: str
    object_id: str
    type: str
    label: str

    def __init__(self, value: str, object_id: str, type: str, label: str):
        self.type = type
        self.object_id = object_id
        self.value = value
        self.label = label

    def __str__(self):
        return f"TeamleaderCustomField label is {self.label} and value is {self.value}"

    def __repr__(self):
        return (
            f"{type(self).__name__}(type= {self.type}, "
            f"value= {self.value}, label= {self.label})"
        )


class TeamleaderCustomFields(dict):
    # custom_fields: Mapping[str, TeamleaderCustomField]

    def __init__(self, custom_fields, get_teamleader, *arg, **kw):
        super(TeamleaderCustomFields, self).__init__(*arg, **kw)
        self._get_teamleader = get_teamleader
        self._make_custom_fiels_readable(custom_fields)

    def _make_custom_fiels_readable(self, custom_fields_local):
        custom_field_file_handler = TeamleaderFileHandler(
            "Custom_fields",
            self._refresh_function,
        )
        custom_fields_info = custom_field_file_handler.file["file"]
        custom_field_id_dict = {
            custom_field_info["id"]: custom_field_info["label"]
            for custom_field_info in custom_fields_info
        }
        self.custom_fields = {}
        for custom_field in custom_fields_local:
            self.custom_fields.update(
                self._make_custom_field_dict_object(custom_field, custom_field_id_dict)
            )

    def _make_custom_field_dict_object(self, custom_field, custom_field_id_dict):
        object_id = custom_field["definition"]["id"]
        label = custom_field_id_dict[object_id]
        return {
            label: TeamleaderCustomField(
                type=custom_field["definition"]["type"],
                object_id=object_id,
                value=custom_field["value"],
                label=label,
            )
        }

    def _refresh_function(self):
        return self._get_teamleader("customFieldDefinitions.list").json()["data"]

    def get(self, custom_field_id):
        return self.custom_fields.get(custom_field_id)


class TeamleaderInfo:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def info(self, object_id, old=True):
        result = self.get(self.url + ".info", data={"id": object_id}).json()["data"]
        if not old:
            result["custom_fields"] = self._transform_custom_fields(result)
        return result

    def _transform_custom_fields(self, result):
        custom_fields = result.get("custom_fields")
        if not custom_fields:
            return TeamleaderCustomFields([], self.get)
        return TeamleaderCustomFields(custom_fields, self.get)


class TeamleaderAdd:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def add(self, name, data={}):
        data["name"] = name
        return self.post(self.url + ".add", data=json.dumps(data))


class TeamleaderUpdate:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def update(self, object_id, data={}):
        data["id"] = object_id
        result = self.get(self.url + ".info", data={"id": object_id}).json()["data"]
        custom_fields = result.get("custom_fields")
        custom_fiels_dict = dict(
            map(lambda item: (item["definition"]["id"], item["value"]), custom_fields)
        )
        for item in data.get("custom_fields", []):
            item_id = item["id"]
            item_value = item["value"]
            custom_fiels_dict[item_id] = item_value

        data["custom_fields"] = [
            {"id": key, "value": value}
            for key, value in custom_fiels_dict.items()
            if value is not None
        ]
        return self.post(self.url + ".update", data=json.dumps(data))


class TeamleaderDelete:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def delete(self, object_id):
        return self.post(self.url + ".delete", data={"id": object_id})


class TeamleaderCreate:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def create(self, data={}):
        return self.post(self.url + ".create", data=json.dumps(data))


class TeamleaderCancel:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def cancel(self, object_id):
        return self.post(
            self.url + ".cancel",
            data=json.dumps({"id": object_id}),
        )


class TeamleaderComplete:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def complete(self, object_id):
        return self.post(
            self.url + ".complete",
            data=json.dumps({"id": object_id}),
        )


class TeamleaderSchedule:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def schedule(self, object_id, starts_at, ends_at):
        return self.post(
            self.url + ".schedule",
            data=json.dumps(
                {"id": object_id, "starts_at": starts_at, "ends_at": ends_at}
            ),
        )


class TeamleaderRegister:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def register(self, register_url: str, types: list):
        if not isinstance(types, list):
            raise TypeError(
                f"Types should be of the type list and is of type {type(types)}"
            )
        types_not_in_webhooks = list(
            filter(lambda webhook_type: not isinstance(webhook_type, Webhook), types)
        )
        if types_not_in_webhooks:
            raise ValueError(f"{types_not_in_webhooks} not exists in the webhook")
        return self.post(
            self.url + ".register",
            data=json.dumps({"url": register_url, "types": types}),
        )


class TeamleaderUnregister:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def unregister(self, register_url: str, types: list):
        if not isinstance(types, list):
            raise TypeError(
                f"Types should be of the type list and is of type {type(types)}"
            )
        types_not_in_webhooks = list(filter(lambda x: x not in Webhook, types))
        if types_not_in_webhooks:
            raise ValueError(f"{types_not_in_webhooks} not exists in the webhook")
        return self.post(
            self.url + ".unregister",
            data=json.dumps({"url": register_url, "types": types}),
        )


class TeamleaderReopen:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def reopen(self):
        raise NotImplementedError("Reopen is not yet implementated")


class TeamleaderAddParticipant:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def add_participant(self):
        raise NotImplementedError("AddParticipant is not yet implementated")


class TeamleaderUpdateParticipant:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def update_participant(self):
        raise NotImplementedError("UpdateParticipant is not yet implementated")


class TeamleaderClose:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def close(self):
        raise NotImplementedError("Close is not yet implementated")


class TeamleaderMove:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def move(self):
        raise NotImplementedError("Move is not yet implemented")


class TeamleaderWin:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def win(self):
        raise NotImplementedError("Win is not yet implemented")


class TeamleaderLose:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.get = get_teamleader
        self.post = post_teamleader

    def lose(self):
        raise NotImplementedError("Lose is not yet implemented")



import enum
from enum import Enum


@enum.unique
class Webhook(str, Enum):
    ACCOUNT_DEACTIVATED = "account.deactivated"
    ACCOUNT_DELETED = "account.deleted"
    COMPANY_ADDED = "company.added"
    COMPANY_DELETED = "company.deleted"
    COMPANY_UPDATED = "company.updated"
    CONTACT_ADDED = "contact.added"
    CONTACT_DELETED = "contact.deleted"
    CONTACT_LINKEDTOCOMPANY = "contact.linkedToCompany"
    CONTACT_UNLINKEDFROMCOMPANY = "contact.unlinkedFromCompany"
    CONTACT_UPDATED = "contact.updated"
    CREDITNOTE_BOOKED = "creditNote.booked"
    DEAL_CREATED = "deal.created"
    DEAL_DELETED = "deal.deleted"
    DEAL_LOST = "deal.lost"
    DEAL_MOVED = "deal.moved"
    DEAL_UPDATED = "deal.updated"
    DEAL_WON = "deal.won"
    INVOICE_BOOKED = "invoice.booked"
    INVOICE_DELETED = "invoice.deleted"
    INVOICE_DRAFTED = "invoice.drafted"
    INVOICE_PAYMENTREGISTERED = "invoice.paymentRegistered"
    INVOICE_PAYMENTREMOVED = "invoice.paymentRemoved"
    INVOICE_SENT = "invoice.sent"
    INVOICE_UPDATED = "invoice.updated"
    MILESTONE_CREATED = "milestone.created"
    MILESTONE_UPDATED = "milestone.updated"
    PRODUCT_ADDED = "product.added"
    PROJECT_CREATED = "project.created"
    PROJECT_DELETED = "project.deleted"
    PROJECT_UPDATED = "project.updated"
    TIMETRACKING_ADDED = "timeTracking.added"
    TIMETRACKING_DELETED = "timeTracking.deleted"
    TIMETRACKING_UPDATED = "timeTracking.updated"
    USER_DEACTIVATED = "user.deactivated"


@enum.unique
class MigrateTypes(str, Enum):
    ACCOUNT = "account"
    USER = "user"
    PRODUCT = "product"
    CONTACT = "contact"
    COMPANY = "company"
    DEAL = "deal"
    DEALPHASE = "dealPhase"
    PROJECT = "project"
    MILESTONE = "milestone"
    TASK = "task"
    MEETING = "meeting"
    CALL = "call"
    TICKET = "ticket"
    INVOICE = "invoice"
    CREDITNOTE = "creditNote"
    SUBSCRIPTION = "subscription"
    QUOTATION = "quotation"



def get_custom_field_value(custom_fields, custom_field_id):
    return get_custom_field(custom_fields, custom_field_id)["value"]


def get_custom_field(custom_fields, custom_field_id):
    for custom_field in custom_fields:
        if custom_field["definition"]["id"] == custom_field_id:
            return custom_field



from .teamleaderMethods import *
from .teamleaderStores import MigrateTypes


class Departments(TeamleaderSimpleList, TeamleaderInfo):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "departments"
        super().__init__(get_teamleader, post_teamleader)


class Users(TeamleaderPagesList, TeamleaderInfo):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "users"
        super().__init__(get_teamleader, post_teamleader)

    # def me(self):
    #     pass


class Projects(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderCreate,
    TeamleaderUpdate,
    TeamleaderClose,
    TeamleaderReopen,
    TeamleaderDelete,
    TeamleaderAddParticipant,
    TeamleaderUpdateParticipant,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "projects"
        super().__init__(get_teamleader, post_teamleader)


class CustomFields(TeamleaderPagesList, TeamleaderInfo):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "customFieldDefinitions"
        super().__init__(get_teamleader, post_teamleader)


class WorkTypes(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "workTypes"
        super().__init__(get_teamleader, post_teamleader)


class Contacts(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderAdd,
    TeamleaderDelete,
    TeamleaderUpdate,
    TeamleaderTag,
    TeamleaderUntag,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "contacts"
        super().__init__(get_teamleader, post_teamleader)
        # self.tag = TeamleaderTag(get_teamleader, post_teamleader)

    # def link_to_company(self):
    #     pass
    #
    # def unlink_from_company(self):
    #     pass


class Companies(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderAdd,
    TeamleaderDelete,
    TeamleaderUpdate,
    TeamleaderTag,
    TeamleaderUntag,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "companies"
        super().__init__(get_teamleader, post_teamleader)
        # self.tag = TeamleaderTag(get_teamleader, post_teamleader)


class BusinessTypes(TeamleaderSimpleList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "businessTypes"
        super().__init__(get_teamleader, post_teamleader)


class Tags(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "tags"
        super().__init__(get_teamleader, post_teamleader)


class DealPhases(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "dealPhases"
        super().__init__(get_teamleader, post_teamleader)


class DealSources(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "dealSources"
        super().__init__(get_teamleader, post_teamleader)


class Tasks(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderCreate,
    TeamleaderUpdate,
    TeamleaderDelete,
    TeamleaderComplete,
    TeamleaderSchedule,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "tasks"
        super().__init__(get_teamleader, post_teamleader)

    #
    # def reopen(self):
    #     pass
    #
    # def schedule(self):
    #     pass


class Invoices(TeamleaderPagesList, TeamleaderInfo):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "invoices"
        super().__init__(get_teamleader, post_teamleader)


class Events(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderCreate,
    TeamleaderUpdate,
    TeamleaderCancel,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "events"
        super().__init__(get_teamleader, post_teamleader)


class Webhooks(TeamleaderSimpleList, TeamleaderRegister, TeamleaderUnregister):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "webhooks"
        super().__init__(get_teamleader, post_teamleader)


class TimeTracking(
    TeamleaderInfo,
    TeamleaderPagesList,
    TeamleaderAdd,
    TeamleaderUpdate,
    TeamleaderDelete,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "timeTracking"
        super().__init__(get_teamleader, post_teamleader)


class Migrate:
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "migrate"
        self.get = get_teamleader

    def get_id(self, old_type: MigrateTypes, old_id: int):
        data = {"type": old_type, "id": old_id}
        response = self.get(self.url + ".id", data=json.dumps(data))
        return response.json()["data"]["id"]


class Deals(
    TeamleaderPagesList,
    TeamleaderInfo,
    TeamleaderCreate,
    TeamleaderUpdate,
    TeamleaderMove,
    TeamleaderWin,
    TeamleaderLose,
    TeamleaderDelete,
):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "deals"
        self.get = get_teamleader
        super().__init__(get_teamleader, post_teamleader)


class LostReasons(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "lostReasons"
        self.get = get_teamleader


class Teams(TeamleaderPagesList):
    def __init__(self, get_teamleader, post_teamleader) -> None:
        self.url = "teams"
        self.get = get_teamleader



from distutils.core import setup

setup(
    name="teamleader",
    packages=["teamleader"],
    version="1.0.5",
    license="MIT",
    description="A layer on top of the Teamleader v2 API",
    author="Jaap",
    author_email="jaap1@me.com",
    url="https://github.com/HeroPP/Teamleader",
    download_url="https://github.com/HeroPP/Teamleader/archive/refs/tags/v1.0.5.tar.gz",
    keywords=["Teamleader", "API", "CRM"],
    install_requires=[
        "certifi",
        "charset-normalizer",
        "idna",
        "oauthlib",
        "pickle5",
        "ratelimit",
        "requests",
        "requests-oauthlib",
        "urllib3",
    ],
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        # Pick your license as you wish
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
    ],
)
